%
% `biharm.m'
%
% I. We first compute a particular solution to 
%
% (1) Delta^2 W = f in B = (-1,1) x (-1,1), 
%
% We thus approximate all three of U, U_x 
% and U_y at the Sinc points in the 
% interior of B.  This is accomplished via 
% use of Sinc convolution, which is a 
% special Sinc technique for approximating 
% the integral 
%
% (2) U(x,y) = int int_B G(x-x', y-y') .
%                . f(x',y') dx' dy'.
%
% We also evaluate this integral 
% directly, via use of Sinc quadrature, 
% over the four rectangles, (-1,x) x (-1,y),
% (-1,x) x (y,1), (x,1) x (-1,y) and 
% (x,1) x (y,1), for comparison. The 
% interested reader should note that Sinc 
% convolution is about two orders of 
% magnitude faster than Sinc quadrature, 
% for approximating this integral.
%
% Our solution U takes the form of (2),  
% with 
%
%  G(x,y) = 1/(16 pi) (x^2 + y^2) log(x^2 + y^2)
%
% where 
%
% f(x,y) = 
%  = 256/81(r^4 - 6 r^2 +18)/(2-r^2)^(8/3)
%
% Such an f is generated by Delta^2 V = f 
%
% with V = (2-r^2)^{4/3}
%
%
% II. Next, we compute a solution, V, of 
% the equation 
%
% (3) Delta^2 V = 0  in B, 
% 
% subject to the conditions 
%
% (4) V = g , V_n = h  
% 
% on the boundary of B.  Our computations are 
% based on the model
%
% (6)  V = real{bar[(z-c)] E + F},
%        = (x-a) u + (y-b) v + phi
%
% weher, E = u + i v, and F = phi + i psi 
% are analytic in B, and where 
% c = a + i b, with a and b real constants.
%
% Then 
%
% (7) Delta^2 V = 0  in B,
%
% We thus solve for u, v and phi on the 
% boundary of B, given  V = g,  V_n = h on 
% the boundary of B.
%
% Specifically, we take a = b, and 
%
% E = (1+i-z)^(1/2), F = z^4.
%
% We can thus deduce, with 
%
%    R = ((1-x)^2 + (1-y)^2)^(1/2),
% 
% that 
% 
% u = ((R + 1-x)/2)^(1/2), 
%
% v = ((R - 1+x)/2)^(1/2).
%
% u_x = -u/(2 R) = v_y, 
%
% v_x = v/(2 R) = -u_x.
% 
% phi = x^3-3xy^2
%
% phi_x = 3 x^3 - 3 y^2
%
% phi_y = -6xy
%
% We will also use the expressions
%
% u1 = u_{Gamma_1} = ((1-y)/2)^{1/2} 
%    = v1, u1_x = -1/(4u1) = -v1_x, 
% phi1 = 1-3y^2, phi1_x = 3(1-y^2),
% g1 = (1-a)u1 + (y-a)v1 + phi1
% h1 = u1 + (1-a)u1_x + (y-a)v1_x + phi1_x
%
% u2 = (1-x)^{1/2},   v2 = 0, phi2 = x^3-3x,
% 
% d = d(x) = (1 + ((1-x)/2)^2)^{1/2}
% u2_y = 0, v2_y = -1/(2u2), phi2_y = -6x, 
% g2 = (x-a)u2 + (1-a)v2 + phi2,
% h2 = (x-a)u2_y + v2 + (1-a)v2_y + phi2_y
% 
% d = d(y) = (1 + ((1-y)/2)^2)^{1/2}
% u3 = (1 + d(y))^{1/2},   v3 = u1^2/(2 d(y) u3)
% phi3 = -phi1, u3_x = -u3/(4d(y)), 
% v3_x = u1^2/(4 u3 d(y)), phi3_x = phi1_x,
% g3 =  (-1-a)u3 + (y-a) v3 + phi3
% h3 = -{ u3 + (-1-a)u3_x + (y-a) v3_x + phi3_x}
%
% u4 = (d(x) + u1^2)^{1/2},  v4 = 1/(2 d(x) u4)
% phi4 = phi2, u4_y = -v4/2, v4_y = u4/(4 d(x)),
% phi4_y = 6x,
% g4 = (x-a) u4 + (-1-a)v4 + phi4,
% h4 = -{(x-a) u4_y + v4 + (-1-a)v4_y + phi4_y}.
% 
% III. Finally, having computed u, v and phi on 
% the boundary of B, we "continue these harmonic 
% funtions into the interior of B via use of a 
% a Sinc procedure.
%
clear all
N = input('Enter number N for Sinc points:   ')
h = 0.8/sqrt(N);
m = 2*N+1;
format long e
%
a = 3;
T = exp(h);
TT = 1/T^N;
for j=1:m
    t(j) = TT;
    optt = 1+TT;
    twott = 2/optt;
    zm(j) = twott; % 1-z(j)
    zp(j) = TT*twott; % 1+z(j)
    w(j) = 2*h*TT/optt^2;
    z(j) = (TT-1)/(TT+1);
    TT = T*TT;
end
%
% In the following  J  is the unit matrix 
% with the order of the rows reversed.
%
J = zeros(m);
%
for j=1:m
    J(j,m+1-j) = 1;
end
%
% Set up of matrices.
%
% Construction and diagonalization of matrices 
% P = I1p & Q = I1m in Sec. 2.1.  Here I1p and 
% I1m are the indefinite integral matrices for 
% integration over (-1,1) from -1 to x and from 
% x to 1 respectively.
%
I1p = Int2_p(-1,1,N,N,h);
I1m = Int2_m(-1,1,N,N,h);
[X S] = eig(I1p);
[Y TTT] = eig(I1m);
s = diag(S);
clear TTT
clear I1m
% 
% Computing the "Laplace transforms"
% hat{G}, hat{Gx}, and hat{Gy} as 
% well as the right hand side, f, 
% see Eqs. (2.5)-(2.8) of the paper.
%   
gam = 0.5772156649015328; % Euler's const.
%
%  The function G below is computed in 
%  the algorithm of  
%  ~/books/toolbox/linux/lap_tr_bihar2*
%
% Timing
%
tic
%
for k=1:m
   vv = s(k);
   teru = vv^3*(2*log(vv) + 3 - 2*gam)/(8*pi);
   for j=1:m
      fr(j,k) = fff(z(j),z(k));
      uu = s(j);
      GG = lap_tr_bihar2(uu,vv);
      G(j,k) = GG;
      Gx(j,k) = GG/uu - teru;
    end
end
%
Gy = Gx.';
% 
% Computing the Sinc Convolution of the 
% Green's Function  G  and its gradient
% 
% This yields a particular solution of 
% the non-homogeneous PDE .
%
WW = biharm_conv(m,X,Y,G,Gx,Gy,fr);
U = WW(1:m,1:m);
UX = WW(1:m,m+1:2*m);
UY = WW(1:m,2*m+1:3*m);
%
toc
%
clear s
clear GG
clear G
clear Gx
clear Gy
%
 surf(z,z,U);
 pause
 surf(z,z,UX);
 pause
 surf(z,z,UY);
 pause
%
tic
% Now, for the "exact" sulution, via use of 
% repeated Sinc quadrature, over the 4 
% rectangles, (-1,z_j)x(-1,z_k), 
%             (-1,z_j)x(z_k,1),
%             (z_j,1)x(-1,z_k), and 
%             (z_j,1)x(z_k,1)
%
for j=1:m
    p = t(j);
    p1 = p+1;
    for k=1:m
        q = t(k);
        q1 = q+1;
        pq1 = p1*q1;
        d1(j,k) = 2*p*q/pq1;
        x1(j,k) = (p*(1-q)-q1)/pq1;
        w1(j,k) = 2*p*q*h/q1/pq1;
        d2(j,k) = -2*q/pq1;
        x2(j,k) = (p*q1-(1-q))/pq1;
        w2(j,k) = 2*q*h/q1/pq1;
    end
end
%
for j=1:m
    for k=1:m
        U_ex(j,k) = 0;
        UX_ex(j,k) = 0;
        UY_ex(j,k) = 0;
        for jj=1:m
            dx1 = d1(j,jj);
            xx1 = x1(j,jj);
            wu1 = w1(j,jj);
            dx2 = d2(j,jj);
            xx2 = x2(j,jj);
            wu2 = w2(j,jj);
            for kk=1:m
                dy1 = d1(k,kk);
                yy1 = x1(k,kk);
                wv1 = w1(k,kk);
                dy2 = d2(k,kk);
                yy2 = x2(k,kk);
                wv2 = w2(k,kk);
                H11 = fff(xx1,yy1)*wu1*wv1*Gr(dx1,dy1);
                H12 = fff(xx1,yy2)*wu1*wv2*Gr(dx1,dy2);
                H21 = fff(xx2,yy1)*wu2*wv1*Gr(dx2,dy1);
                H22 = fff(xx2,yy2)*wu2*wv2*Gr(dx2,dy2);
                U_ex(j,k) = U_ex(j,k) + H11(1)+H12(1)+H21(1)+H22(1);
                UX_ex(j,k) = UX_ex(j,k) + H11(2)+H12(2)+H21(2)+H22(2);
                UY_ex(j,k) = UY_ex(j,k) + H11(3)+H12(3)+H21(3)+H22(3);
            end
        end
    end
end
%
% Let us now plot the differences between the 
% two sets of solutions computed above.
%
toc
surf(z,z,U - U_ex);
pause
%
surf(z,z,UX - UX_ex);
pause
%
surf(z,z,UY - UY_ex);
pause
% We next need to compute a solution of the 
% homeogenous PDE to satisfy the boundary 
% conditions.  This is done via BIE, but in 
% the present program we are using Sinc 
% quadrature over (-1,1).  We will set up 
% the block matrix system, which contains 
% (2L)^2 blocks re L=4 arcs.  We set up the 
% right hand side vector for each arc, then 
% the block matrices.  
%
% Following this, we also construct the marix
% blocks.
%
% Constructing right-hand side vectors, g & h
%
% Notice the negatie signs re h2 and h4, since 
% we are doing outward normals.
%
for j=1:m
    rt2 = sqrt(2);
    r = z(j);
    s = zm(j);
    a1 = sqrt(s/2);
    d = sqrt(1 + s^2/4);
    e = sqrt(d + s^2);
    f = sqrt(d + 1);
    u1(j) = a1;
    v1(j) = a1;
    p1(j) = 1 - 3*r^2;
    u1x = -1/4/a1;
    v1x = -u1x;
    p1x = 3*s*zp(j);
    g1(j) = (1-a)*a1 + (r-a)*a1 + p1(j);
    h1(j) = a1 + (1-a)*u1x + (r-a)*v1x + p1x;
    %
    a2 = sqrt(s);
    u2(j) = a2;
    v2(j) = 0;
    p2(j) = r^3-3*r;
    u2y = 0;
    v2y = -1/2/a2;
    p2y = -6*r;
    g2(j) = (r-a)*a2 + p2(j);
    h2(j) = (1-a)*v2y + p2y;
    %
    u3(j) = f;
    v3(j) = a1^2/2/d/f;
    p3(j) = -p1(j);
    u3x = -f/4/d;
    v3x = a1^2/4/f/d;
    p3x = p1x;
    g3(j) = (-1-a)*u3(j) + (r-a)*v3(j) + p3(j);
    h3(j) = -(u3(j) + (-1-a)*u3x + (r-a)*v3x + p3x);
    %
    u4(j) = e;
    v4(j) = 1/2/e/d;
    p4(j) = p2(j);
    u4y = -1/4/d/e;
    v4y = e/4/d;
    p4y = 6*r;
    g4(j) = (r-a)*u4(j) + (-1-a)*v4(j) + p4(j);
    h4(j) = -((r-a)*u4y + v4(j) + (-1-a)*v4y + p4y);
end
%
% We must now list the vectors on arcs Gamma2 and 
% Gamma4 in reverse order, since these are oriented.
%
te = J*u2.';
u2 = te.';
te = J*v2.';
v2 = te.';
te = J*g2.';
g2 = te.';
te = J*p2.';
p2 = te.';
te = J*h2.';
h2 = te.';
te = J*u3.';
u3 = te.';
te = J*v3.';
v3 = te.';
te = J*p3.';
p3 = te.';
te = J*g3.';
g3 = te.';
te = J*h3.';
h3 = te.';
%
gg = [g1 g2 g3 g4].';
hh = [h1 h2 h3 h4].';
%
%
% The Matrices P^l of (3.9)
%
for j=1:m
    ta = zp(j);
    tb = zm(j);
    for k=1:m
        za = zp(k); 
        zb = zm(k);
        zc = z(j) + z(k);
        v = w(k);
        P2(j,k) = tb*v/pi/(tb^2 + za^2);
        P3(j,k) = 2*v/pi/(4 + zc^2);
        P4(j,k) = ta*v/pi/(ta^2 + zb^2);
    end
end
%
% The Hilbert transform matrix
%
O = zeros(m);
H = [O P2 P3 P4; ...
      P4 O P2 P3; ...
      P3 P4 O P2; ...
      P2 P3 P4 O];
%
% Solving the homogeneous biharmonic 
% BVP
%
Ug = bih_D_homo(a,m,z,w,P2,P3,P4,gg,hh);
% 
%
% First, our approximation u on Gamma
%
    u1a = Ug(1:m);
    u2a = Ug(m+1:2*m);
    u3a = Ug(2*m+1:3*m);
    u4a = Ug(3*m+1:4*m);
%
    phi1a = Ug(4*m+1:5*m);
    phi2a = Ug(5*m+1:6*m);
    phi3a = Ug(6*m+1:7*m);
    phi4a = Ug(7*m+1:8*m);
%
% Then, the conjugate function, v:
%
UUg = [u1a.'  u2a.' u3a.' u4a.'].';
Vg = H*UUg;
%
% and ..
%
    v1a = Vg(1:m);
    v2a = Vg(m+1:2*m);
    v3a = Vg(2*m+1:3*m);
    v4a = Vg(3*m+1:4*m);
%
% Testing values on the contours:
%
% First, on contour Gamma_1:
%
plot(z,u1a,'o',z,u1,'-')
pause
plot(z,v1a,'o',z,v1,'-')
pause
plot(z,phi1a,'o',z,p1,'-')
pause
%
% Analytic continuation of the solution to the 
% interior of B
%
% We first set up the matrices Kl as defined in (4.9).
% Note here, that 
%
% K^2 = [theta^2_k(z^1(t_j))] = [theta^4_k(z^3(t_j))]
% K^3 = [theta^3_k(z^1(t_j))] = [theta^4_k(z^2(t_j))]
% 
for j=1:m
    z1 = 1 + i*z(j);
    pp4 = log((z1+1+i)/(1-i-z1));
    z2 = -z(j) + i;
    pp3 = log((z2+1+i)/(1-i-z2));
    z3 = -1-i*z(j);
    pp2 = log((z3+1+i)/(1-i-z3));
    K2(j,:) = basis_harm(pp2,N,N,h);
    K3(j,:) = basis_harm(pp3,N,N,h);
    K4(j,:) = basis_harm(pp4,N,N,h);
end
%
% Now form the matrix in (4.9)
%
II = eye(m);
E = [II K2 K3 K4; ...
     K4 II K2 K3; ...
     K3 K4 II K2; ...
     K2 K3 K4 II];
 %
 % Analytic continuation of u:
 %
 u = [u1a.' u2a.' u3a.' u4a.'].';
 v = [v1a.' v2a.' v3a.' v4a.'].';
 ph = [phi1a.' phi2a.' phi3a.' phi4a.'].';
 cu = E\u;
 cv = E\v;
 cp = E\ph;
 cu1 = cu(1:m);
 cu2 = cu(m+1:2*m);
 cu3 = cu(2*m+1:3*m);
 cu4 = cu(3*m+1:4*m);
 cv1 = cv(1:m);
 cv2 = cv(m+1:2*m);
 cv3 = cv(2*m+1:3*m);
 cv4 = cv(3*m+1:4*m);
 cp1 = cp(1:m);
 cp2 = cp(m+1:2*m);
 cp3 = cp(2*m+1:3*m);
 cp4 = cp(3*m+1:4*m);
 %
 %%%%
 %%%% The Following is in error, & needs corrections! 
 %%%%
 u = zeros(m);
 v = zeros(m);
 phi = zeros(m);
 for j=1:m
     for k=1:m
         xx = z(j) + i*z(k);
         ph1 = log((xx - (1-i))/(1+i-xx));
         ph2 = log((xx-(1+i))/(-1+i-xx));
         ph3 = log((xx+1-i)/(-1-i-xx));
         ph4 = log((xx+1+i)/(1-i-xx));
         uu1 = basis_harm(ph1,N,N,h);
         uu2 = basis_harm(ph2,N,N,h);
         uu3 = basis_harm(ph3,N,N,h);
         uu4 = basis_harm(ph4,N,N,h);
         u(j,k) = uu1*cu1 + uu2*cu2 + uu3*cu3 + uu4*cu4;
         v(j,k) = uu1*cv1 + uu2*cv2 + uu3*cv3 + uu4*cv4;
         phi(j,k) = uu1*cp1 + uu2*cp2 + uu3*cp3 + uu4*cp4;
         VV(j,k) = (z(j)-a)*u(j,k) + (z(k)-a)*v(j,k) + phi(j,k);
         RR = sqrt(zm(j)^2 + zm(k)^2);
         u_ex(j,k) = sqrt((RR + zm(j))/2);
      end
 end
 %
 surf(z,z,u);
 pause
 surf(z,z,u_ex);
 pause
 surf(z,z,u_ex-u);
 pause
 surf(z,z,VV);
 pause
 surf(z,z,U_ex);
 pause
 surf(z,z,VV-U_ex);
             
         
 
 
 

        

