%cp 
% 'lap_harm_disc.m
%
% We solve here the problem 
%
% nabla u = 0  in D
%
% with D the unit disc, and 
% subject to the boundary conditins
%
% u = 1 on C_+, u = -1 on C_-
%
% where C_+ (resp., C_-) denotes 
% the boundary of D in the upper 
% (resp., lower) half of the 
% complex plane.
%
% The exact solution to this problem
% is
% 
% u =  2/pi*tan^{-1}(2y/(1-x^2-y^2)).
%  
% The method of solution of via use 
% of the harmonic Sinc basis defined 
% in Sec. 1.5.13, the implementation 
% of which is illustrated in Sec. 4.1.1.
% We thus firs solve a 2 x 2 block 
% system of equations 
%
%    [ I  B ] c1 = `1'
%    [ C  I ] c2 = `-1'
%
% we determine the vector (c1 , c2)^T, 
% and we use the expression 
%
% U = sum_j [c1_j del1,j(z) + c2_j del2(j,z)
%
% to compute the solution in the 
% interior of D.
%
% First compute the vectors c1 & c2:
clear all
N = 20;
h = 3/sqrt(N);
m = 2*N+1;
z1 = Sincpoints7(0,pi,N,N,h);
z2 = Sincpoints7(pi,2*pi,N,N,h);
B = zeros(m,m);
C = B;
E = eye(m);
%
for j=1:m
    C(j,:) = basis_harm7(pi,2*pi,z1(j),N,N,h);
    b1(j) = 1;
    B(j,:) = basis_harm7(0,pi,z2(j),N,N,h);
    b2(j) = -1;
end
%
Mat = [E B;C E];
b = [b1 b2].';
%
sol = Mat\b;
%
c1 = real(sol(1:m));
c2 = real(sol(m+1:2*m));
%
% Next compute the solution in the 
% interior of D.  However, we will plot 
% only the part in the upper half plane,
% since the suolution in the lower half 
% is just the negative (with respect to 
% the vertical coordinate) of that in the 
% lower half plane
%
h2 = 0.9;
T = exp(h2);
TT = 1/T^N;
for j=1:m
   rt = sqrt(TT);
   den = 1 + TT;
   x(j) = (TT-1)/den;
   r(j) = 2*rt/den;
   TT = T*TT;
end
%
T = exp(h2);
TT = 1/T^N;
for k=1:m
    y(k) = TT/(1+TT);
    TT = T*TT;
end
%
U = zeros(m,m);
%
for j=1:m
    for k=1:m
        xx = x(j);
        yy = r(j)*y(k);
        zz = xx + i*yy;
        v1 = basis_harm7(0,pi,zz,N,N,h);
        v2 = basis_harm7(pi,2*pi,zz,N,N,h);
        U(j,k) = v1*c1 + v2*c2;
        u_exact(j,k) = 2/pi*atan(2*yy/(1-xx^2-yy^2));
     end
end
%
% Now, for some plots:
%
figure(1)
surf(y,x,U);
pause
figure(2)
surf(y,x,u_exact);
pause
figure(3)
surf(y,x,U - u_exact);
