%
% Program:  `p_d_picard.m'
%
% This program is copyrighted by SINC, LLC
% an belongs to SINC, LLC
% 
% F(x,t) = F_0(x) + U(x,t) - V(x,t) 
%
%   U(x,t) = int_0^t int_x^infinity G(x,u) b(u) "times" 
%               "times" F_x(u,tau) du dtau 
%
%   V(x,t) = int_0^t mu(tau) int_0^x F_x(v,tau) "times" 
%     "times" int_{x-v}^infinity w^{-1} a(v,w) "times" 
%      "times" F_x(w,tau) dw dv dtau
%
% A successive approximation solution over 
% (0,infinity) x (0,infinity) is computed.
% See Sec. 4.4
%
clear all
format long e
%
% Initial value inputs
%
% Note  M1 slighly bigger than N1, to take care 
% of the boundary layer.
%
% Begin timing
%
tic
M1 = 10; 
N1 = 8;
m1 = M1 + N1 + 1;
M2 = 8;
N2 = 6;
m2 = M2 + N2 + 1;
ht = 0.4/sqrt(N2);
hx = 0.7/sqrt(N1);
% T = 1;
%
format long e
%
% Sinc Points, Weights, and Matrices
%
x = Sincpoints3(M1,N1,hx);
wx = weights3(M1,N1,hx);
%
% t = Sincpoints2(0,T,M2,N2,ht);
t = Sincpoints3(M2,N2,ht);
%
% Integration over (0,x) and (x,infinity)
%
Axp = Int3_p(M1,N1,hx);
Axm = Int3_m(M1,N1,hx);
%
% The derivative matrices
%
% The "x" derivative matrix on (0,infinity)
% Note the many possibilities of Sinc 
% differentiation on (0, infinity).
%
Axd = Dpoly3_1(M1,N1,hx);
%
% Integration over (0,t)
%
% Atp = Int2_p(0,T,M2,N2,ht);
Atp = Int3_p(M2,N2,ht);
%
% First, some definitions of functions and 
%
% some diagonal matrices.
%
for j=1:m1
%    
% the initial F_0, b
%
F0(j) = 1 - exp(- x(j)/0.5); 
b(j) = 1/(1+x(j)^2); % b(x) = 1/(1+x^2)
for k=1:m1
%
% The next two are values of G and a 
%
 G(j,k) = x(j)/(x(j)^2 + x(k)^2+1); % x/(x^2 + y^2 + 1)
 a(j,k) = x(j)/(x(j)^2 + x(k)^2+1); % x/(x^2 + y^2 + 1)
end
end
%
% The diagonals of full matrices
%
for j=1:m1
    for k=1:m2
%
% The initial value of F
%
        FF0(j,k) = F0(j);
    end
end
%
% The approximation of "mu"
%
for k=1:m2
    mu(k) = 1/(1+t(k)^2);
end
%
Dmu = diag(mu);
%
% Start of iterative "contraction"  loop
% 
F = FF0;  % FF0 = F(x,t)|_{t=0}
nn = 1 
%
% The value of the "least squares"
% convergence norm, is here set to 1
%
while(nn > 0.00001)
%
% First compute the U(x,t) part -- see (4.3.5)
%
% First, compute the U = U(x,t) - Part
%
% F_x denoted by Fx
%
Fx = Axd * F;
%
% Next, G(x,u) b(u) F_x(u,tau)
%
for j=1:m1
    for k=1:m2
        for l=1:m1
        GbFx(j,k,l) = G(j,l) * b(l) * Fx(l,k);
        end
    end
end
%
% P2(x,tau) = int_x^infinity G(x,u) b(u) F_x(u,tau) du
%
for j=1:m1
    for k=1:m2
        P2(j,k) = 0;
        for l=1:m1
            P2(j,k) = P2(j,k) + Axm(j,l) * GbFx(j,k,l);
        end
    end
end
%
clear GbFx
%
% We now get U(x,t) at Sinc pts.:
% U(x,t) = int_0^t P2(x,tau) dtau
%
U = P2 * Atp.';
% 
clear P2
%
%
%  V(x,t) is next:
%
for k=1:m2
    for l=1:m1
        Fxx(l) = F(l,k);
        for j=1:m1
            awFx(j,l) = a(j,l)/x(l)*Fxx(l); 
        % awFx(v,w,tau) = a(v,w)/w*F_x(w,tau)
        end
    end
%
% Set ker_p(v,y,tau) = int_y^infinity awFx(v,w,tau) dw
%   
    ker_p = Axm*awFx;
    p = real(conv_nl_discrete_p(x,x,wx,Axp,ker_p));
    for l=1:m1
        p(l) = Fxx(l)*p(l);
    end
    PP(k,l) = p(l);
end
%
% At this stage we have 
% int_0^x F_x(v,tau) int_{x-v}^infty (a(v,w)/w) F(w,tau).
%
% We next multiply this by mu(tau) and integrate the result from 
% 0 to t:
%
V = PP.'*Dmu*Atp.';
% 
% We have thus completed the computation for V
%
% Next form the difference FF = F - FF0 - U + V:
%
FF = F - FF0 - U + V;
%
W = real(FF);
%
F = F - W;
nn = norm(W);
end
%
% end timing
toc
%
surf(t,x,F)
pause
print -dps2 p_d_picard.ps
